@@ -21,6 +21,8 @@
 #ifndef __BF_LIB_H_
 #define __BF_LIB_H_
 
+#include <time.h> /* for time_t */
+
 typedef struct {
 	gint my_int;
 	gchar *my_char;
@@ -33,6 +35,35 @@ typedef enum {
 	tcc2i_full_match_gettext
 } Ttcc2i_mode;
 
+typedef struct {
+	gpointer data;
+	gint count;
+} Trefcpointer;
+
+/* #define REFP_DEBUG */
+Trefcpointer *refcpointer_new(gpointer data);
+#ifdef REFP_DEBUG
+void refcpointer_ref(Trefcpointer *rp);
+#else
+#define refcpointer_ref(rp) rp->count++
+#endif
+void refcpointer_unref(Trefcpointer *rp);
+
+GFile *add_suffix_to_uri(GFile *file, const char *suffix);
+const gchar *full_path_utf8_from_uri(GFile *uri);
+gchar *filename_utf8_from_uri(GFile *uri);
+
+GList *urilist_to_stringlist(GList *urilist);
+void free_urilist(GList *urilist);
+#if GTK_CHECK_VERSION(2,14,0)
+#else
+GFile *gtk_file_chooser_get_file(GtkFileChooser *chooser);
+#endif
+/*#define user_bfdir(file) g_strconcat(g_get_home_dir(), "/."PACKAGE"/", file, NULL)*/
+GFile *user_bfdir(const gchar *filename);
+
+gchar *filename_utf8_from_full_path_utf8(const gchar *full_path_utf8);
+
 gchar *get_filename_on_disk_encoding(const gchar *utf8filename);
 gchar *get_utf8filename_from_on_disk_encoding(const gchar *encodedname);
 gboolean string_is_color(const gchar *color);
@@ -43,6 +74,7 @@ void list_switch_order(GList *first, GList *second);
 gboolean file_copy(gchar *source, gchar *dest);
 gint find_common_prefixlen_in_stringlist(GList *stringlist);
 gboolean append_string_to_file(gchar *filename, gchar *string);
+/*guint countchars(const gchar *string, const gchar *chars);*/
 gint table_convert_char2int(Tconvert_table *table, const gchar *my_char, Ttcc2i_mode mode);
 gchar *table_convert_int2char(Tconvert_table *table, gint my_int);
 gchar *expand_string(const gchar *string, const char specialchar, Tconvert_table *table);
@@ -56,24 +88,36 @@ void free_convert_table(Tconvert_table *tct);
 #define utf8_byteoffset_to_charsoffset(string,byteoffset) g_utf8_pointer_to_offset(string, string+byteoffset)
 /*glong utf8_byteoffset_to_charsoffset(gchar *string, glong byteoffset);*/
 void utf8_offset_cache_reset();
-guint utf8_byteoffset_to_charsoffset_cached(gchar *string, glong byteoffset);
+guint utf8_byteoffset_to_charsoffset_cached(const gchar *string, glong byteoffset);
 
 gchar *strip_any_whitespace(gchar *string);
 gchar *trunc_on_char(gchar * string, gchar which_char);
 gchar *strip_common_path(char *to_filename, char *from_filename);
 gchar *most_efficient_filename(gchar *filename);
 gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepath);
-gchar *create_full_path(const gchar * filename, const gchar *basedir);
+
+gchar *strip_trailing_slash(gchar *input);
 gchar *ending_slash(const gchar *dirname);
 gchar *path_get_dirname_with_ending_slash(const gchar *filename);
-gboolean file_exists_and_readable(const gchar * filename);
-gchar *return_first_existing_filename(const gchar * filename, ...);
-gboolean filename_test_extensions(gchar **extensions, gchar *filename);
+gboolean full_path_exists(const gchar *full_path);
+/*gboolean file_exists_and_readable(const gchar * filename);*/
+GFile *return_first_existing_filename(const gchar * filename, ...);
+gboolean filename_test_extensions(gchar **extensions, const gchar *filename);
 gchar *bf_str_repeat(const gchar * str, gint number_of);
-gint get_int_from_string(gchar *string);
-gchar *create_secure_dir_return_filename();
-void remove_secure_dir_and_filename(gchar *filename);
+gint get_int_from_string(const gchar *string);
+gchar *unique_path(const gchar *basedir, const gchar *prefix, GHashTable *excludehash);
+gchar *create_secure_dir_return_filename(void);
+/*void remove_secure_dir_and_filename(gchar *filename);*/
 /*gchar *buf_replace_char(gchar *buf, gint len, gchar srcchar, gchar destchar);*/
 void wordcount(gchar *text, guint *chars, guint *lines, guint *words);
+/*GSList *gslist_from_glist_reversed(GList *src);*/
 GList *glist_from_gslist(GSList *src);
+gchar *bf_portable_time(const time_t *timep);
+
+gboolean arr3_equal (gconstpointer v1,gconstpointer v2);
+guint arr3_hash(gconstpointer v);
+gchar *gfile_display_name(GFile *uri,GFileInfo *finfo);
+gboolean gfile_uri_is_parent(GFile *parent, GFile *child, gboolean recursive);
+gchar *get_hostname_from_uri(GFile *uri);
+
 #endif /* __BF_LIB_H_ */
