@@ -23,24 +23,27 @@
 #define __STRINGLIST_H_
 
 #include <glib.h>
-
+#include <gio/gio.h>
 /*void estrl_dialog(GList **which_list, gchar *title, gint what_list
 				, gint column_num, gchar **column_titles, void (*post_dialog_func)());*/
 
-gint count_array(gchar **array);
+/*gint count_array(gchar **array);*/
+#define count_array g_strv_length
 gchar *array_to_string(gchar **array);
 gchar **string_to_array(gchar *string);
 gchar **array_from_arglist(const gchar *string1, ...);
 GList *list_from_arglist(gboolean allocate_strings, ...);
 GList *duplicate_stringlist(GList *list, gint dup_data);
 gint free_stringlist(GList * which_list);
-GList *get_list(const gchar * filename, GList * which_list, gboolean is_arraylist);
-GList *get_stringlist(const gchar * filename, GList * which_list);
-gboolean put_stringlist_limited(gchar * filename, GList * which_list, gint maxentries);
-gboolean put_stringlist(gchar * filename, GList * which_list);
+GList *get_list_from_buffer(gchar *buffer, GList *which_list, gboolean is_arraylist);
+GList *get_list(GFile* file, GList * which_list, gboolean is_arraylist);
+GList *get_stringlist(GFile *file, GList * which_list);
+gboolean put_stringlist_limited(GFile * file, GList * which_list, gint maxentries, gboolean is_arraylist);
+gboolean put_stringlist(GFile * file, GList * which_list, gboolean is_arraylist);
 
 gint free_arraylist(GList * which_list);
-gchar **duplicate_stringarray(gchar **array);
+/*gchar **duplicate_stringarray(gchar **array);*/
+#define duplicate_stringarray g_strdupv
 GList *duplicate_arraylist(GList *arraylist);
 
 /* removes a string from a stringlist if it is the same */
@@ -51,12 +54,11 @@ GList *add_to_history_stringlist(GList *which_list, const gchar *string, gboolea
 GList *add_to_stringlist(GList * which_list, const gchar * string);
 gchar *stringlist_to_string(GList *stringlist, gchar *delimiter);
 
-gint array_n_strings_identical(gchar **array1, gchar **array2, gboolean case_sensitive, gint testlevel);
-GList *arraylist_delete_identical(GList *arraylist, gchar **compare, gint testlevel, gboolean case_sensitive);
-GList *arraylist_append_identical_from_list(GList *thelist, GList *source, gchar **compare, gint testlevel, gboolean case_sensitive);
-GList *arraylist_append_identical_from_file(GList *thelist, const gchar *sourcefilename, gchar **compare, gint testlevel, gboolean case_sensitive);
-
-gboolean arraylist_value_exists(GList *arraylist, gchar **value, gint testlevel, gboolean case_sensitive);
+gint array_n_strings_identical(const gchar **array1, const gchar **array2, gboolean case_sensitive, gint testlevel);
+GList *arraylist_delete_identical(GList *arraylist, const gchar **compare, gint testlevel, gboolean case_sensitive);
+GList *arraylist_append_identical_from_list(GList *thelist, GList *source, const gchar **compare, gint testlevel, gboolean case_sensitive);
+GList *arraylist_append_identical_from_file(GList *thelist, const gchar *sourcefilename, const gchar **compare, gint testlevel, gboolean case_sensitive);
+gchar **arraylist_value_exists(GList *arraylist, const gchar **value, gint testlevel, gboolean case_sensitive);
 GList *arraylist_load_new_identifiers_from_list(GList *mylist, GList *deflist, gint uniquelevel);
 GList *arraylist_load_new_identifiers_from_file(GList *mylist, const gchar *fromfilename, gint uniquelevel);
 #endif							/* __STRINGLIST_H_ */
